import os

import config
from scopeton.decorators import Inject


class DepUtil:
    @Inject()
    def __init__(self, conf: config.ArgsConfig):
        self.conf = conf

    def resolve_full_dir(self, dep_config: config.FileConfig, dep: config.Dependency):
        return os.path.join(dep_config.dep_path, self.conf.lib_path, dep.path)


import os.path
from dataclasses import dataclass


@dataclass
class Dep:
    path: str
    git_url: str
    repo_dir: str
    skip: bool

    def full_dir(self):
        return os.path.join(self.path, self.repo_dir)



import subprocess


def call(cwd, process_with_params, throw_on_error=True):
    ret_code = subprocess.call(process_with_params, cwd=cwd)
    if throw_on_error and ret_code != 0:
        raise Exception("Error executing[{}]: {}".format(cwd, process_with_params))
    return ret_code



#!/usr/bin/env python
import logging
import argparse
import os.path

from scopeton.scope import Scope

import config
import dep_util
import dependency_resolver
import git_util
from config import ArgsConfig

parser = argparse.ArgumentParser(description='Openscad package manager')
parser.add_argument('--v', action='store_true', help='verbose output')
parser.add_argument('--upgrade', action='store_true', help='upgrade packages')
parser.add_argument('--lib-path', dest='lib_path', default='./lib', help='library path')
parser.add_argument('--pm-file', dest='pm_file', help='pm file name')

args = parser.parse_args()
args_config = ArgsConfig()
args_config.__dict__ = vars(args)
logging.basicConfig(level=logging.DEBUG if args.v else logging.INFO)

scope = Scope()
scope.registerInstance(ArgsConfig, args_config)
scope.registerBean(
    config.YamlConfigParser,
    config.MainConfigParser,
    config.PmConfigParser,
    dep_util.DepUtil,
    dependency_resolver.DependencyResolver,
    git_util.GitFetcher
)

resolver = scope.getInstance(dependency_resolver.DependencyResolver)
resolver.run("./", args_config.pm_file)



import logging
import os

from scopeton.decorators import Inject

import config
import dep_util
import proc_util


class GitFetcher:
    @Inject()
    def __init__(self, conf: config.ArgsConfig, dep_util: dep_util.DepUtil):
        self.conf = conf
        self.dep_util = dep_util
    def _resolve_lib_path(self, dep_config: config.FileConfig, dep: config.Dependency):
        return os.path.join(dep_config.dep_path or "", self.conf.lib_path)

    def fetch(self, dep_config: config.FileConfig, dep: config.Dependency):
        if not os.path.exists(self._resolve_lib_path(dep_config, dep)):
            os.mkdir(self._resolve_lib_path(dep_config, dep))
        logging.info("Full dir: {}".format(self.dep_util.resolve_full_dir(dep_config, dep)))
        if not os.path.exists(self.dep_util.resolve_full_dir(dep_config, dep)):
            self.download_dep(dep_config, dep)
        else:
            self.update_dep(dep_config, dep)

    def update_dep(self, dep_config: config.FileConfig, dep: config.Dependency):
        if not self.conf.upgrade:
            self.goto_revision(dep_config, dep)
            logging.info("skipping upgrade, see --upgrade, for package: {}".format(dep))
            return
        logging.info("Updating dep: {}".format(dep))
        proc_util.call(self.dep_util.resolve_full_dir(dep_config, dep), ["git", "pull"])
        self.goto_revision(dep_config, dep)

    def download_dep(self, dep_config: config.FileConfig, dep: config.Dependency):
        logging.info("Downloading dep: {}".format(dep))
        proc_util.call(self._resolve_lib_path(dep_config, dep), ["git", "clone", dep.url])
        self.goto_revision(dep_config, dep)

    def goto_revision(self, dep_config: config.FileConfig, dep: config.Dependency):
        if dep.revision is not None:
            logging.info("Checking out the revision: {}".format(dep.revision))
            proc_util.call(self.dep_util.resolve_full_dir(dep_config, dep), ["git", "checkout", dep.revision])





import logging
import os

from scopeton.decorators import Inject
from scopeton.scope import Scope

import config
import dep_util
import git_util


class DependencyResolver:
    @Inject()
    def __init__(self, scope: Scope, conf: config.ArgsConfig, parser: config.MainConfigParser,
                 fetcher: git_util.GitFetcher, dep_util: dep_util.DepUtil):
        self.conf = conf
        self.parser = parser
        self.scope = scope
        self.fetcher = fetcher
        self.dep_util = dep_util

    def _check_type(self, path: str, file: str):
        if os.path.exists(os.path.join(path, file)):
            return file
        else:
            return None

    def _resolve_file_name(self, path: str, *args):
        for k in args:
            logging.info("trying: {} with {}".format(path, k))
            res = self._check_type(path, k)
            if res:
                return res

    def _do_resolve_deps(self, dep_config: config.FileConfig):
        logging.info("resolving dependencies: {}".format(dep_config))
        for dep in dep_config.dependencies:
            self._resolve_dep(dep_config, dep)

    def _resolve_dep(self, dep_config: config.FileConfig, dep: config.Dependency):
        logging.info("Resolving dependency: {}".format(dep))
        self.fetcher.fetch(dep_config, dep)
        file = self._resolve_file_name(self.dep_util.resolve_full_dir(dep_config, dep), "scad_pm.yaml", "scad.pm")
        if file:
            self.run(self.dep_util.resolve_full_dir(dep_config, dep), file)

    def run(self, path, file=None):
        logging.info("Running with path: {} and file: {}".format(path, file))
        if file is None:
            file = self._resolve_file_name(path, "scad_pm.yaml", "scad.pm")
        if file is not None:
            parsed = self.parser.parse(os.path.join(path, file))
            parsed.dep_path = path
            self._do_resolve_deps(parsed)
        else:
            logging.info("No dependency files found in {}".format(path))



import setuptools
from setuptools import setup

setup(
    name='scad_pm',
    version='0.22',
    scripts=['src/scad_pm'],
    install_requires=[
        'pyyaml',
        'dataclasses_json',
        'scopeton'
   ]
)


import abc
import dataclasses
import logging
import typing

import scopeton.scope
from dataclasses_json import dataclass_json
import yaml

# @dataclasses.dataclass
# @dataclass_json
from scopeton.decorators import Inject


class ArgsConfig:
    lib_path: str
    upgrade: bool
    v: bool
    pm_file: str

    def __str__(self):
        return "{}:[{}]".format(self.__class__.__name__, self.__dict__)


class DEP_TYPES:
    git = "git"


@dataclass_json()
@dataclasses.dataclass
class Dependency(object):
    url: str
    path: typing.Optional[str] = None
    revision: typing.Optional[str] = None
    dependency_type: str = DEP_TYPES.git


@dataclass_json()
@dataclasses.dataclass
class FileConfig(object):
    dependencies: typing.List[Dependency]
    dep_path: typing.Union[str, None] = None


class ConfigParser:
    @abc.abstractmethod
    def parse(self, file: str):
        pass

    @abc.abstractmethod
    def is_type(self, file: str) -> bool:
        pass


class YamlConfigParser(ConfigParser):
    def is_type(self, file: str) -> bool:
        return file.split(".")[-1] == 'yaml'

    @Inject()
    def __init__(self, args_config: ArgsConfig):
        self.args_config = args_config

    def parse(self, file: str):
        with open(file, "r") as stream:
            data = yaml.safe_load(stream)
            logging.info("Parsing data: {}".format(data))
            return self.map_data(data)

    def map_data(self, data):
        conf = FileConfig.from_dict(data)  # type: FileConfig
        for dep in conf.dependencies:
            if dep.path is None:
                dep.path = dep.url.split("/")[-1].split(".")[0]
        return conf


class PmConfigParser(ConfigParser):
    def is_type(self, file: str) -> bool:
        return file.split(".")[-1] == 'pm'

    @Inject()
    def __init__(self, args_config: ArgsConfig):
        self.args_config = args_config

    def parse(self, file: str):
        with open(file, "r") as stream:
            data = stream.readlines()
            logging.info("Parsing data: {}".format(data))
            return self.map_data(data)

    def map_data(self, data):
        deps = [self._parse_dep(k) for k in data]
        deps = filter(lambda x: x is not None, deps)
        return FileConfig(list(deps))

    def _parse_dep(self, k):
        k = k.strip()
        if k.startswith("#"):
            return None
        path = k.split("/")[-1].split(".")[0]
        if "#" in k:
            branch = k.split("#")[1]
        else:
            branch = None
        path = path.split("#")[0]
        return Dependency(
            k,
            path=path,
            revision=branch
        )


class MainConfigParser:
    @Inject()
    def __init__(self, scope: scopeton.scope.Scope):
        self.scope = scope
        self.parsers = scope.getInstances(ConfigParser)
        logging.info("Config parsers: {}".format(self.parsers))

    def _get_parser(self, file: str) -> ConfigParser:
        for k in self.parsers:
            if k.is_type(file):
                return k
        raise Exception("Parser not found for file: {}".format(file))

    def parse(self, file: str) -> FileConfig:
        return self._get_parser(file).parse(file)
